// Created on: 1994-02-08
// Created by: Gilles DEBARBOUILLE
// Copyright (c) 1994-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _Quantity_NameOfColor_HeaderFile
#define _Quantity_NameOfColor_HeaderFile

//! Definition of names of known colors.
//! The names come (mostly) from the X11 specification.
enum Quantity_NameOfColor
{
  Quantity_NOC_BLACK,
  Quantity_NOC_MATRABLUE,
  Quantity_NOC_MATRAGRAY,
  Quantity_NOC_ALICEBLUE,
  Quantity_NOC_ANTIQUEWHITE,
  Quantity_NOC_ANTIQUEWHITE1,
  Quantity_NOC_ANTIQUEWHITE2,
  Quantity_NOC_ANTIQUEWHITE3,
  Quantity_NOC_ANTIQUEWHITE4,
  Quantity_NOC_AQUAMARINE1,
  Quantity_NOC_AQUAMARINE2,
  Quantity_NOC_AQUAMARINE4,
  Quantity_NOC_AZURE,
  Quantity_NOC_AZURE2,
  Quantity_NOC_AZURE3,
  Quantity_NOC_AZURE4,
  Quantity_NOC_BEIGE,
  Quantity_NOC_BISQUE,
  Quantity_NOC_BISQUE2,
  Quantity_NOC_BISQUE3,
  Quantity_NOC_BISQUE4,
  Quantity_NOC_BLANCHEDALMOND,
  Quantity_NOC_BLUE,
  Quantity_NOC_BLUE1 = Quantity_NOC_BLUE,
  Quantity_NOC_BLUE2,
  Quantity_NOC_BLUE3,
  Quantity_NOC_BLUE4,
  Quantity_NOC_BLUEVIOLET,
  Quantity_NOC_BROWN,
  Quantity_NOC_BROWN1,
  Quantity_NOC_BROWN2,
  Quantity_NOC_BROWN3,
  Quantity_NOC_BROWN4,
  Quantity_NOC_BURLYWOOD,
  Quantity_NOC_BURLYWOOD1,
  Quantity_NOC_BURLYWOOD2,
  Quantity_NOC_BURLYWOOD3,
  Quantity_NOC_BURLYWOOD4,
  Quantity_NOC_CADETBLUE,
  Quantity_NOC_CADETBLUE1,
  Quantity_NOC_CADETBLUE2,
  Quantity_NOC_CADETBLUE3,
  Quantity_NOC_CADETBLUE4,
  Quantity_NOC_CHARTREUSE,
  Quantity_NOC_CHARTREUSE1 = Quantity_NOC_CHARTREUSE,
  Quantity_NOC_CHARTREUSE2,
  Quantity_NOC_CHARTREUSE3,
  Quantity_NOC_CHARTREUSE4,
  Quantity_NOC_CHOCOLATE,
  Quantity_NOC_CHOCOLATE1,
  Quantity_NOC_CHOCOLATE2,
  Quantity_NOC_CHOCOLATE3,
  Quantity_NOC_CHOCOLATE4,
  Quantity_NOC_CORAL,
  Quantity_NOC_CORAL1,
  Quantity_NOC_CORAL2,
  Quantity_NOC_CORAL3,
  Quantity_NOC_CORAL4,
  Quantity_NOC_CORNFLOWERBLUE,
  Quantity_NOC_CORNSILK1,
  Quantity_NOC_CORNSILK2,
  Quantity_NOC_CORNSILK3,
  Quantity_NOC_CORNSILK4,
  Quantity_NOC_CYAN,
  Quantity_NOC_CYAN1 = Quantity_NOC_CYAN,
  Quantity_NOC_CYAN2,
  Quantity_NOC_CYAN3,
  Quantity_NOC_CYAN4,
  Quantity_NOC_DARKGOLDENROD,
  Quantity_NOC_DARKGOLDENROD1,
  Quantity_NOC_DARKGOLDENROD2,
  Quantity_NOC_DARKGOLDENROD3,
  Quantity_NOC_DARKGOLDENROD4,
  Quantity_NOC_DARKGREEN,
  Quantity_NOC_DARKKHAKI,
  Quantity_NOC_DARKOLIVEGREEN,
  Quantity_NOC_DARKOLIVEGREEN1,
  Quantity_NOC_DARKOLIVEGREEN2,
  Quantity_NOC_DARKOLIVEGREEN3,
  Quantity_NOC_DARKOLIVEGREEN4,
  Quantity_NOC_DARKORANGE,
  Quantity_NOC_DARKORANGE1,
  Quantity_NOC_DARKORANGE2,
  Quantity_NOC_DARKORANGE3,
  Quantity_NOC_DARKORANGE4,
  Quantity_NOC_DARKORCHID,
  Quantity_NOC_DARKORCHID1,
  Quantity_NOC_DARKORCHID2,
  Quantity_NOC_DARKORCHID3,
  Quantity_NOC_DARKORCHID4,
  Quantity_NOC_DARKSALMON,
  Quantity_NOC_DARKSEAGREEN,
  Quantity_NOC_DARKSEAGREEN1,
  Quantity_NOC_DARKSEAGREEN2,
  Quantity_NOC_DARKSEAGREEN3,
  Quantity_NOC_DARKSEAGREEN4,
  Quantity_NOC_DARKSLATEBLUE,
  Quantity_NOC_DARKSLATEGRAY1,
  Quantity_NOC_DARKSLATEGRAY2,
  Quantity_NOC_DARKSLATEGRAY3,
  Quantity_NOC_DARKSLATEGRAY4,
  Quantity_NOC_DARKSLATEGRAY,
  Quantity_NOC_DARKTURQUOISE,
  Quantity_NOC_DARKVIOLET,
  Quantity_NOC_DEEPPINK,
  Quantity_NOC_DEEPPINK2,
  Quantity_NOC_DEEPPINK3,
  Quantity_NOC_DEEPPINK4,
  Quantity_NOC_DEEPSKYBLUE1,
  Quantity_NOC_DEEPSKYBLUE2,
  Quantity_NOC_DEEPSKYBLUE3,
  Quantity_NOC_DEEPSKYBLUE4,
  Quantity_NOC_DODGERBLUE1,
  Quantity_NOC_DODGERBLUE2,
  Quantity_NOC_DODGERBLUE3,
  Quantity_NOC_DODGERBLUE4,
  Quantity_NOC_FIREBRICK,
  Quantity_NOC_FIREBRICK1,
  Quantity_NOC_FIREBRICK2,
  Quantity_NOC_FIREBRICK3,
  Quantity_NOC_FIREBRICK4,
  Quantity_NOC_FLORALWHITE,
  Quantity_NOC_FORESTGREEN,
  Quantity_NOC_GAINSBORO,
  Quantity_NOC_GHOSTWHITE,
  Quantity_NOC_GOLD,
  Quantity_NOC_GOLD1 = Quantity_NOC_GOLD,
  Quantity_NOC_GOLD2,
  Quantity_NOC_GOLD3,
  Quantity_NOC_GOLD4,
  Quantity_NOC_GOLDENROD,
  Quantity_NOC_GOLDENROD1,
  Quantity_NOC_GOLDENROD2,
  Quantity_NOC_GOLDENROD3,
  Quantity_NOC_GOLDENROD4,
  Quantity_NOC_GRAY,
  Quantity_NOC_GRAY0,
  Quantity_NOC_GRAY1,
  Quantity_NOC_GRAY2,
  Quantity_NOC_GRAY3,
  Quantity_NOC_GRAY4,
  Quantity_NOC_GRAY5,
  Quantity_NOC_GRAY6,
  Quantity_NOC_GRAY7,
  Quantity_NOC_GRAY8,
  Quantity_NOC_GRAY9,
  Quantity_NOC_GRAY10,
  Quantity_NOC_GRAY11,
  Quantity_NOC_GRAY12,
  Quantity_NOC_GRAY13,
  Quantity_NOC_GRAY14,
  Quantity_NOC_GRAY15,
  Quantity_NOC_GRAY16,
  Quantity_NOC_GRAY17,
  Quantity_NOC_GRAY18,
  Quantity_NOC_GRAY19,
  Quantity_NOC_GRAY20,
  Quantity_NOC_GRAY21,
  Quantity_NOC_GRAY22,
  Quantity_NOC_GRAY23,
  Quantity_NOC_GRAY24,
  Quantity_NOC_GRAY25,
  Quantity_NOC_GRAY26,
  Quantity_NOC_GRAY27,
  Quantity_NOC_GRAY28,
  Quantity_NOC_GRAY29,
  Quantity_NOC_GRAY30,
  Quantity_NOC_GRAY31,
  Quantity_NOC_GRAY32,
  Quantity_NOC_GRAY33,
  Quantity_NOC_GRAY34,
  Quantity_NOC_GRAY35,
  Quantity_NOC_GRAY36,
  Quantity_NOC_GRAY37,
  Quantity_NOC_GRAY38,
  Quantity_NOC_GRAY39,
  Quantity_NOC_GRAY40,
  Quantity_NOC_GRAY41,
  Quantity_NOC_GRAY42,
  Quantity_NOC_GRAY43,
  Quantity_NOC_GRAY44,
  Quantity_NOC_GRAY45,
  Quantity_NOC_GRAY46,
  Quantity_NOC_GRAY47,
  Quantity_NOC_GRAY48,
  Quantity_NOC_GRAY49,
  Quantity_NOC_GRAY50,
  Quantity_NOC_GRAY51,
  Quantity_NOC_GRAY52,
  Quantity_NOC_GRAY53,
  Quantity_NOC_GRAY54,
  Quantity_NOC_GRAY55,
  Quantity_NOC_GRAY56,
  Quantity_NOC_GRAY57,
  Quantity_NOC_GRAY58,
  Quantity_NOC_GRAY59,
  Quantity_NOC_GRAY60,
  Quantity_NOC_GRAY61,
  Quantity_NOC_GRAY62,
  Quantity_NOC_GRAY63,
  Quantity_NOC_GRAY64,
  Quantity_NOC_GRAY65,
  Quantity_NOC_GRAY66,
  Quantity_NOC_GRAY67,
  Quantity_NOC_GRAY68,
  Quantity_NOC_GRAY69,
  Quantity_NOC_GRAY70,
  Quantity_NOC_GRAY71,
  Quantity_NOC_GRAY72,
  Quantity_NOC_GRAY73,
  Quantity_NOC_GRAY74,
  Quantity_NOC_GRAY75,
  Quantity_NOC_GRAY76,
  Quantity_NOC_GRAY77,
  Quantity_NOC_GRAY78,
  Quantity_NOC_GRAY79,
  Quantity_NOC_GRAY80,
  Quantity_NOC_GRAY81,
  Quantity_NOC_GRAY82,
  Quantity_NOC_GRAY83,
  Quantity_NOC_GRAY85,
  Quantity_NOC_GRAY86,
  Quantity_NOC_GRAY87,
  Quantity_NOC_GRAY88,
  Quantity_NOC_GRAY89,
  Quantity_NOC_GRAY90,
  Quantity_NOC_GRAY91,
  Quantity_NOC_GRAY92,
  Quantity_NOC_GRAY93,
  Quantity_NOC_GRAY94,
  Quantity_NOC_GRAY95,
  Quantity_NOC_GRAY97,
  Quantity_NOC_GRAY98,
  Quantity_NOC_GRAY99,
  Quantity_NOC_GREEN,
  Quantity_NOC_GREEN1 = Quantity_NOC_GREEN,
  Quantity_NOC_GREEN2,
  Quantity_NOC_GREEN3,
  Quantity_NOC_GREEN4,
  Quantity_NOC_GREENYELLOW,
  Quantity_NOC_HONEYDEW,
  Quantity_NOC_HONEYDEW2,
  Quantity_NOC_HONEYDEW3,
  Quantity_NOC_HONEYDEW4,
  Quantity_NOC_HOTPINK,
  Quantity_NOC_HOTPINK1,
  Quantity_NOC_HOTPINK2,
  Quantity_NOC_HOTPINK3,
  Quantity_NOC_HOTPINK4,
  Quantity_NOC_INDIANRED,
  Quantity_NOC_INDIANRED1,
  Quantity_NOC_INDIANRED2,
  Quantity_NOC_INDIANRED3,
  Quantity_NOC_INDIANRED4,
  Quantity_NOC_IVORY,
  Quantity_NOC_IVORY2,
  Quantity_NOC_IVORY3,
  Quantity_NOC_IVORY4,
  Quantity_NOC_KHAKI,
  Quantity_NOC_KHAKI1,
  Quantity_NOC_KHAKI2,
  Quantity_NOC_KHAKI3,
  Quantity_NOC_KHAKI4,
  Quantity_NOC_LAVENDER,
  Quantity_NOC_LAVENDERBLUSH1,
  Quantity_NOC_LAVENDERBLUSH2,
  Quantity_NOC_LAVENDERBLUSH3,
  Quantity_NOC_LAVENDERBLUSH4,
  Quantity_NOC_LAWNGREEN,
  Quantity_NOC_LEMONCHIFFON1,
  Quantity_NOC_LEMONCHIFFON2,
  Quantity_NOC_LEMONCHIFFON3,
  Quantity_NOC_LEMONCHIFFON4,
  Quantity_NOC_LIGHTBLUE,
  Quantity_NOC_LIGHTBLUE1,
  Quantity_NOC_LIGHTBLUE2,
  Quantity_NOC_LIGHTBLUE3,
  Quantity_NOC_LIGHTBLUE4,
  Quantity_NOC_LIGHTCORAL,
  Quantity_NOC_LIGHTCYAN,
  Quantity_NOC_LIGHTCYAN1 = Quantity_NOC_LIGHTCYAN,
  Quantity_NOC_LIGHTCYAN2,
  Quantity_NOC_LIGHTCYAN3,
  Quantity_NOC_LIGHTCYAN4,
  Quantity_NOC_LIGHTGOLDENROD,
  Quantity_NOC_LIGHTGOLDENROD1,
  Quantity_NOC_LIGHTGOLDENROD2,
  Quantity_NOC_LIGHTGOLDENROD3,
  Quantity_NOC_LIGHTGOLDENROD4,
  Quantity_NOC_LIGHTGOLDENRODYELLOW,
  Quantity_NOC_LIGHTGRAY,
  Quantity_NOC_LIGHTPINK,
  Quantity_NOC_LIGHTPINK1,
  Quantity_NOC_LIGHTPINK2,
  Quantity_NOC_LIGHTPINK3,
  Quantity_NOC_LIGHTPINK4,
  Quantity_NOC_LIGHTSALMON1,
  Quantity_NOC_LIGHTSALMON2,
  Quantity_NOC_LIGHTSALMON3,
  Quantity_NOC_LIGHTSALMON4,
  Quantity_NOC_LIGHTSEAGREEN,
  Quantity_NOC_LIGHTSKYBLUE,
  Quantity_NOC_LIGHTSKYBLUE1,
  Quantity_NOC_LIGHTSKYBLUE2,
  Quantity_NOC_LIGHTSKYBLUE3,
  Quantity_NOC_LIGHTSKYBLUE4,
  Quantity_NOC_LIGHTSLATEBLUE,
  Quantity_NOC_LIGHTSLATEGRAY,
  Quantity_NOC_LIGHTSTEELBLUE,
  Quantity_NOC_LIGHTSTEELBLUE1,
  Quantity_NOC_LIGHTSTEELBLUE2,
  Quantity_NOC_LIGHTSTEELBLUE3,
  Quantity_NOC_LIGHTSTEELBLUE4,
  Quantity_NOC_LIGHTYELLOW,
  Quantity_NOC_LIGHTYELLOW2,
  Quantity_NOC_LIGHTYELLOW3,
  Quantity_NOC_LIGHTYELLOW4,
  Quantity_NOC_LIMEGREEN,
  Quantity_NOC_LINEN,
  Quantity_NOC_MAGENTA,
  Quantity_NOC_MAGENTA1 = Quantity_NOC_MAGENTA,
  Quantity_NOC_MAGENTA2,
  Quantity_NOC_MAGENTA3,
  Quantity_NOC_MAGENTA4,
  Quantity_NOC_MAROON,
  Quantity_NOC_MAROON1,
  Quantity_NOC_MAROON2,
  Quantity_NOC_MAROON3,
  Quantity_NOC_MAROON4,
  Quantity_NOC_MEDIUMAQUAMARINE,
  Quantity_NOC_MEDIUMORCHID,
  Quantity_NOC_MEDIUMORCHID1,
  Quantity_NOC_MEDIUMORCHID2,
  Quantity_NOC_MEDIUMORCHID3,
  Quantity_NOC_MEDIUMORCHID4,
  Quantity_NOC_MEDIUMPURPLE,
  Quantity_NOC_MEDIUMPURPLE1,
  Quantity_NOC_MEDIUMPURPLE2,
  Quantity_NOC_MEDIUMPURPLE3,
  Quantity_NOC_MEDIUMPURPLE4,
  Quantity_NOC_MEDIUMSEAGREEN,
  Quantity_NOC_MEDIUMSLATEBLUE,
  Quantity_NOC_MEDIUMSPRINGGREEN,
  Quantity_NOC_MEDIUMTURQUOISE,
  Quantity_NOC_MEDIUMVIOLETRED,
  Quantity_NOC_MIDNIGHTBLUE,
  Quantity_NOC_MINTCREAM,
  Quantity_NOC_MISTYROSE,
  Quantity_NOC_MISTYROSE2,
  Quantity_NOC_MISTYROSE3,
  Quantity_NOC_MISTYROSE4,
  Quantity_NOC_MOCCASIN,
  Quantity_NOC_NAVAJOWHITE1,
  Quantity_NOC_NAVAJOWHITE2,
  Quantity_NOC_NAVAJOWHITE3,
  Quantity_NOC_NAVAJOWHITE4,
  Quantity_NOC_NAVYBLUE,
  Quantity_NOC_OLDLACE,
  Quantity_NOC_OLIVEDRAB,
  Quantity_NOC_OLIVEDRAB1,
  Quantity_NOC_OLIVEDRAB2,
  Quantity_NOC_OLIVEDRAB3,
  Quantity_NOC_OLIVEDRAB4,
  Quantity_NOC_ORANGE,
  Quantity_NOC_ORANGE1 = Quantity_NOC_ORANGE,
  Quantity_NOC_ORANGE2,
  Quantity_NOC_ORANGE3,
  Quantity_NOC_ORANGE4,
  Quantity_NOC_ORANGERED,
  Quantity_NOC_ORANGERED1 = Quantity_NOC_ORANGERED,
  Quantity_NOC_ORANGERED2,
  Quantity_NOC_ORANGERED3,
  Quantity_NOC_ORANGERED4,
  Quantity_NOC_ORCHID,
  Quantity_NOC_ORCHID1,
  Quantity_NOC_ORCHID2,
  Quantity_NOC_ORCHID3,
  Quantity_NOC_ORCHID4,
  Quantity_NOC_PALEGOLDENROD,
  Quantity_NOC_PALEGREEN,
  Quantity_NOC_PALEGREEN1,
  Quantity_NOC_PALEGREEN2,
  Quantity_NOC_PALEGREEN3,
  Quantity_NOC_PALEGREEN4,
  Quantity_NOC_PALETURQUOISE,
  Quantity_NOC_PALETURQUOISE1,
  Quantity_NOC_PALETURQUOISE2,
  Quantity_NOC_PALETURQUOISE3,
  Quantity_NOC_PALETURQUOISE4,
  Quantity_NOC_PALEVIOLETRED,
  Quantity_NOC_PALEVIOLETRED1,
  Quantity_NOC_PALEVIOLETRED2,
  Quantity_NOC_PALEVIOLETRED3,
  Quantity_NOC_PALEVIOLETRED4,
  Quantity_NOC_PAPAYAWHIP,
  Quantity_NOC_PEACHPUFF,
  Quantity_NOC_PEACHPUFF2,
  Quantity_NOC_PEACHPUFF3,
  Quantity_NOC_PEACHPUFF4,
  Quantity_NOC_PERU,
  Quantity_NOC_PINK,
  Quantity_NOC_PINK1,
  Quantity_NOC_PINK2,
  Quantity_NOC_PINK3,
  Quantity_NOC_PINK4,
  Quantity_NOC_PLUM,
  Quantity_NOC_PLUM1,
  Quantity_NOC_PLUM2,
  Quantity_NOC_PLUM3,
  Quantity_NOC_PLUM4,
  Quantity_NOC_POWDERBLUE,
  Quantity_NOC_PURPLE,
  Quantity_NOC_PURPLE1,
  Quantity_NOC_PURPLE2,
  Quantity_NOC_PURPLE3,
  Quantity_NOC_PURPLE4,
  Quantity_NOC_RED,
  Quantity_NOC_RED1 = Quantity_NOC_RED,
  Quantity_NOC_RED2,
  Quantity_NOC_RED3,
  Quantity_NOC_RED4,
  Quantity_NOC_ROSYBROWN,
  Quantity_NOC_ROSYBROWN1,
  Quantity_NOC_ROSYBROWN2,
  Quantity_NOC_ROSYBROWN3,
  Quantity_NOC_ROSYBROWN4,
  Quantity_NOC_ROYALBLUE,
  Quantity_NOC_ROYALBLUE1,
  Quantity_NOC_ROYALBLUE2,
  Quantity_NOC_ROYALBLUE3,
  Quantity_NOC_ROYALBLUE4,
  Quantity_NOC_SADDLEBROWN,
  Quantity_NOC_SALMON,
  Quantity_NOC_SALMON1,
  Quantity_NOC_SALMON2,
  Quantity_NOC_SALMON3,
  Quantity_NOC_SALMON4,
  Quantity_NOC_SANDYBROWN,
  Quantity_NOC_SEAGREEN,
  Quantity_NOC_SEAGREEN1,
  Quantity_NOC_SEAGREEN2,
  Quantity_NOC_SEAGREEN3,
  Quantity_NOC_SEAGREEN4,
  Quantity_NOC_SEASHELL,
  Quantity_NOC_SEASHELL2,
  Quantity_NOC_SEASHELL3,
  Quantity_NOC_SEASHELL4,
  Quantity_NOC_BEET,
  Quantity_NOC_TEAL,
  Quantity_NOC_SIENNA,
  Quantity_NOC_SIENNA1,
  Quantity_NOC_SIENNA2,
  Quantity_NOC_SIENNA3,
  Quantity_NOC_SIENNA4,
  Quantity_NOC_SKYBLUE,
  Quantity_NOC_SKYBLUE1,
  Quantity_NOC_SKYBLUE2,
  Quantity_NOC_SKYBLUE3,
  Quantity_NOC_SKYBLUE4,
  Quantity_NOC_SLATEBLUE,
  Quantity_NOC_SLATEBLUE1,
  Quantity_NOC_SLATEBLUE2,
  Quantity_NOC_SLATEBLUE3,
  Quantity_NOC_SLATEBLUE4,
  Quantity_NOC_SLATEGRAY1,
  Quantity_NOC_SLATEGRAY2,
  Quantity_NOC_SLATEGRAY3,
  Quantity_NOC_SLATEGRAY4,
  Quantity_NOC_SLATEGRAY,
  Quantity_NOC_SNOW,
  Quantity_NOC_SNOW2,
  Quantity_NOC_SNOW3,
  Quantity_NOC_SNOW4,
  Quantity_NOC_SPRINGGREEN,
  Quantity_NOC_SPRINGGREEN2,
  Quantity_NOC_SPRINGGREEN3,
  Quantity_NOC_SPRINGGREEN4,
  Quantity_NOC_STEELBLUE,
  Quantity_NOC_STEELBLUE1,
  Quantity_NOC_STEELBLUE2,
  Quantity_NOC_STEELBLUE3,
  Quantity_NOC_STEELBLUE4,
  Quantity_NOC_TAN,
  Quantity_NOC_TAN1,
  Quantity_NOC_TAN2,
  Quantity_NOC_TAN3,
  Quantity_NOC_TAN4,
  Quantity_NOC_THISTLE,
  Quantity_NOC_THISTLE1,
  Quantity_NOC_THISTLE2,
  Quantity_NOC_THISTLE3,
  Quantity_NOC_THISTLE4,
  Quantity_NOC_TOMATO,
  Quantity_NOC_TOMATO1 = Quantity_NOC_TOMATO,
  Quantity_NOC_TOMATO2,
  Quantity_NOC_TOMATO3,
  Quantity_NOC_TOMATO4,
  Quantity_NOC_TURQUOISE,
  Quantity_NOC_TURQUOISE1,
  Quantity_NOC_TURQUOISE2,
  Quantity_NOC_TURQUOISE3,
  Quantity_NOC_TURQUOISE4,
  Quantity_NOC_VIOLET,
  Quantity_NOC_VIOLETRED,
  Quantity_NOC_VIOLETRED1,
  Quantity_NOC_VIOLETRED2,
  Quantity_NOC_VIOLETRED3,
  Quantity_NOC_VIOLETRED4,
  Quantity_NOC_WHEAT,
  Quantity_NOC_WHEAT1,
  Quantity_NOC_WHEAT2,
  Quantity_NOC_WHEAT3,
  Quantity_NOC_WHEAT4,
  Quantity_NOC_WHITESMOKE,
  Quantity_NOC_YELLOW,
  Quantity_NOC_YELLOW1 = Quantity_NOC_YELLOW,
  Quantity_NOC_YELLOW2,
  Quantity_NOC_YELLOW3,
  Quantity_NOC_YELLOW4,
  Quantity_NOC_YELLOWGREEN,
  Quantity_NOC_WHITE
};

#endif // _Quantity_NameOfColor_HeaderFile
