// Created on: 1997-03-26
// Created by: Christian CAILLET
// Copyright (c) 1997-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _StepGeom_CartesianTransformationOperator2d_HeaderFile
#define _StepGeom_CartesianTransformationOperator2d_HeaderFile

#include <Standard.hxx>
#include <Standard_Type.hxx>

#include <StepGeom_CartesianTransformationOperator.hxx>

class StepGeom_CartesianTransformationOperator2d;
DEFINE_STANDARD_HANDLE(StepGeom_CartesianTransformationOperator2d,
                       StepGeom_CartesianTransformationOperator)

//! Added from StepGeom Rev2 to Rev4
class StepGeom_CartesianTransformationOperator2d : public StepGeom_CartesianTransformationOperator
{

public:
  Standard_EXPORT StepGeom_CartesianTransformationOperator2d();

  DEFINE_STANDARD_RTTIEXT(StepGeom_CartesianTransformationOperator2d,
                          StepGeom_CartesianTransformationOperator)

protected:
private:
};

#endif // _StepGeom_CartesianTransformationOperator2d_HeaderFile
