// Created on: 2009-05-10
// Created by: Paul V. SUPRYATKIN
// Copyright (c) 2009-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#define Font_NOF_MONOSPACE "monospace"
#define Font_NOF_SERIF "serif"
#define Font_NOF_SANS_SERIF "sans-serif"
#define Font_NOF_CJK "cjk"       // Font_UnicodeSubset_CJK
#define Font_NOF_KOREAN "korean" // Font_UnicodeSubset_Korean
#define Font_NOF_ARABIC "arabic" // Font_UnicodeSubset_Arabic

#define Font_NOF_ASCII_MONO "Courier"
#define Font_NOF_ASCII_SIMPLEX "Times-Roman"
#define Font_NOF_ASCII_COMPLEX "Times-Roman"
#define Font_NOF_ASCII_DUPLEX "Times-Bold"
#define Font_NOF_ASCII_TRIPLEX "Times-Bold"
#define Font_NOF_ASCII_ITALIC_COMPLEX "Times-Italic"
#define Font_NOF_ASCII_ITALIC_TRIPLEX "Times-BoldItalic"
#define Font_NOF_ASCII_SCRIPT_SIMPLEX "ZapfChancery-MediumItalic"
#define Font_NOF_ASCII_SCRIPT_COMPLEX "ZapfChancery-MediumItalic"
#define Font_NOF_GREEK_MONO "Symbol"
#define Font_NOF_GREEK_SIMPLEX "Symbol"
#define Font_NOF_GREEK_COMPLEX "Symbol"
#define Font_NOF_SYMBOL_MONO "ZapfDingbats"
#define Font_NOF_SYMBOL_SIMPLEX "ZapfDingbats"
#define Font_NOF_CARTOGRAPHIC_SIMPLEX "Rock"
#define Font_NOF_KANJI_MONO "Iris"
#define Font_NOF_KATAKANA_MONO "Iris"
