// Created on: 1994-11-22
// Created by: Laurent BOURESCHE
// Copyright (c) 1994-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _ChFiKPart_ComputeData_Sphere_HeaderFile
#define _ChFiKPart_ComputeData_Sphere_HeaderFile

Standard_Boolean ChFiKPart_Sphere(TopOpeBRepDS_DataStructure&      DStr,
                                  const Handle(ChFiDS_SurfData)&   Data,
                                  const Handle(Adaptor3d_Surface)& S1,
                                  const Handle(Adaptor3d_Surface)& S2,
                                  const TopAbs_Orientation         OrFace1,
                                  const TopAbs_Orientation         OrFace2,
                                  const TopAbs_Orientation         Or1,
                                  const TopAbs_Orientation         Or2,
                                  const Standard_Real              Rad,
                                  const gp_Pnt2d&                  PS1,
                                  const gp_Pnt2d&                  P1S2,
                                  const gp_Pnt2d&                  P2S2);

#endif
