// Created on: 2015-08-06
// Created by: Ilya Novikov
// Copyright (c) 2004-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _XCAFDimTolObjects_DimensionFormVariance_HeaderFile
#define _XCAFDimTolObjects_DimensionFormVariance_HeaderFile

//! Defines value of form variance
enum XCAFDimTolObjects_DimensionFormVariance
{
  XCAFDimTolObjects_DimensionFormVariance_None,
  XCAFDimTolObjects_DimensionFormVariance_A,
  XCAFDimTolObjects_DimensionFormVariance_B,
  XCAFDimTolObjects_DimensionFormVariance_C,
  XCAFDimTolObjects_DimensionFormVariance_CD,
  XCAFDimTolObjects_DimensionFormVariance_D,
  XCAFDimTolObjects_DimensionFormVariance_E,
  XCAFDimTolObjects_DimensionFormVariance_EF,
  XCAFDimTolObjects_DimensionFormVariance_F,
  XCAFDimTolObjects_DimensionFormVariance_FG,
  XCAFDimTolObjects_DimensionFormVariance_G,
  XCAFDimTolObjects_DimensionFormVariance_H,
  XCAFDimTolObjects_DimensionFormVariance_JS,
  XCAFDimTolObjects_DimensionFormVariance_J,
  XCAFDimTolObjects_DimensionFormVariance_K,
  XCAFDimTolObjects_DimensionFormVariance_M,
  XCAFDimTolObjects_DimensionFormVariance_N,
  XCAFDimTolObjects_DimensionFormVariance_P,
  XCAFDimTolObjects_DimensionFormVariance_R,
  XCAFDimTolObjects_DimensionFormVariance_S,
  XCAFDimTolObjects_DimensionFormVariance_T,
  XCAFDimTolObjects_DimensionFormVariance_U,
  XCAFDimTolObjects_DimensionFormVariance_V,
  XCAFDimTolObjects_DimensionFormVariance_X,
  XCAFDimTolObjects_DimensionFormVariance_Y,
  XCAFDimTolObjects_DimensionFormVariance_Z,
  XCAFDimTolObjects_DimensionFormVariance_ZA,
  XCAFDimTolObjects_DimensionFormVariance_ZB,
  XCAFDimTolObjects_DimensionFormVariance_ZC
};

#endif // _XCAFDimTolObjects_DimensionFormVariance_HeaderFile
