// Created on: 1997-10-08
// Created by: Prestataire Xuan PHAM PHU
// Copyright (c) 1997-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _TopOpeBRepTool_defineG_HeaderFile
#define _TopOpeBRepTool_defineG_HeaderFile

#include <Geom_Curve.hxx>
#include <Geom_TrimmedCurve.hxx>
#include <Geom2d_Curve.hxx>
#include <Geom2d_TrimmedCurve.hxx>

#define Mhgc Handle(Geom_Curve)
#define Mhgtc Handle(Geom_TrimmedCurve)
#define Mhg2c Handle(Geom2d_Curve)
#define Mhg2tc Handle(Geom2d_TrimmedCurve)

#define Mgc Geom_Curve
#define Mgtc Geom_TrimmedCurve
#define Mg2c Geom2d_Curve
#define Mg2tc Geom_TrimmedCurve

#endif
