// Created on: 1992-12-16
// Created by: Jean Louis FRENKEL
// Copyright (c) 1992-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _StdPrs_ToolPoint_HeaderFile
#define _StdPrs_ToolPoint_HeaderFile

#include <Standard.hxx>
#include <Standard_DefineAlloc.hxx>
#include <Standard_Handle.hxx>

class Geom_Point;

class StdPrs_ToolPoint
{
public:
  DEFINE_STANDARD_ALLOC

  Standard_EXPORT static void Coord(const Handle(Geom_Point)& aPoint,
                                    Standard_Real&            X,
                                    Standard_Real&            Y,
                                    Standard_Real&            Z);
};

#endif // _StdPrs_ToolPoint_HeaderFile
