#ifndef TKDEIGES_PCH_H
#define TKDEIGES_PCH_H

#include <IGESData_HArray1OfIGESEntity.hxx>
#include <IGESData_IGESEntity.hxx>
#include <IGESData_IGESReaderData.hxx>
#include <IGESData_ParamReader.hxx>

#endif // TKDEIGES_PCH_H
