// Created on: 1991-03-12
// Created by: Michel CHAUVAT
// Copyright (c) 1991-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _GProp_ValueType_HeaderFile
#define _GProp_ValueType_HeaderFile

//! Algorithms:
enum GProp_ValueType
{
  GProp_Mass,
  GProp_CenterMassX,
  GProp_CenterMassY,
  GProp_CenterMassZ,
  GProp_InertiaXX,
  GProp_InertiaYY,
  GProp_InertiaZZ,
  GProp_InertiaXY,
  GProp_InertiaXZ,
  GProp_InertiaYZ,
  GProp_Unknown
};

#endif // _GProp_ValueType_HeaderFile
