// Author: Kirill Gavrilov
// Copyright (c) 2017-2019 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _RWObj_SubMeshReason_HeaderFile
#define _RWObj_SubMeshReason_HeaderFile

//! Reason for creating a new group within OBJ reader.
enum RWObj_SubMeshReason
{
  RWObj_SubMeshReason_NewObject,   //!< new object, should occur only ones in valid OBJ file (at the
                                   //!< very beginning)
  RWObj_SubMeshReason_NewGroup,    //!< new group (g item)
  RWObj_SubMeshReason_NewMaterial, //!< new material (usemtl item)
  RWObj_SubMeshReason_NewSmoothGroup //!< new smoothing group (s item)
};

#endif // _RWObj_SubMeshReason_HeaderFile
