// Created on: 1993-06-17
// Created by: Jean Yves LEBEY
// Copyright (c) 1993-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _TopOpeBRep_HeaderFile
#define _TopOpeBRep_HeaderFile

#include <Standard.hxx>
#include <Standard_DefineAlloc.hxx>
#include <Standard_Handle.hxx>

#include <Standard_OStream.hxx>
#include <TopOpeBRep_TypeLineCurve.hxx>

//! This package provides  the topological  operations
//! on the BRep data structure.
class TopOpeBRep
{
public:
  DEFINE_STANDARD_ALLOC

  //! Prints the name  of  <TLC>  as  a String  on the
  //! Stream <S> and returns <S>.
  Standard_EXPORT static Standard_OStream& Print(const TopOpeBRep_TypeLineCurve TLC,
                                                 Standard_OStream&              OS);
};

#endif // _TopOpeBRep_HeaderFile
