// Created on: 1995-08-23
// Created by: Xavier BENVENISTE
// Copyright (c) 1995-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef Convert_CosAndSinEvalFunction_HeaderFile
#define Convert_CosAndSinEvalFunction_HeaderFile

#include <TColgp_Array1OfPnt2d.hxx>
#include <TColStd_Array1OfReal.hxx>
#include <TColStd_Array1OfInteger.hxx>

typedef void Convert_CosAndSinEvalFunction(Standard_Real,
                                           const Standard_Integer,
                                           const TColgp_Array1OfPnt2d&,
                                           const TColStd_Array1OfReal&,
                                           const TColStd_Array1OfInteger*,
                                           Standard_Real Result[2]);

#endif
