// Created by: DAUTRY Philippe
// Copyright (c) 1997-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _TDF_DefaultDeltaOnRemoval_HeaderFile
#define _TDF_DefaultDeltaOnRemoval_HeaderFile

#include <Standard.hxx>

#include <TDF_DeltaOnRemoval.hxx>
class TDF_Attribute;

class TDF_DefaultDeltaOnRemoval;
DEFINE_STANDARD_HANDLE(TDF_DefaultDeltaOnRemoval, TDF_DeltaOnRemoval)

//! This class provides a default implementation of a
//! TDF_DeltaOnRemoval.
class TDF_DefaultDeltaOnRemoval : public TDF_DeltaOnRemoval
{

public:
  //! Creates a TDF_DefaultDeltaOnRemoval.
  Standard_EXPORT TDF_DefaultDeltaOnRemoval(const Handle(TDF_Attribute)& anAttribute);

  //! Applies the delta to the attribute.
  Standard_EXPORT virtual void Apply() Standard_OVERRIDE;

  DEFINE_STANDARD_RTTIEXT(TDF_DefaultDeltaOnRemoval, TDF_DeltaOnRemoval)

protected:
private:
};

#endif // _TDF_DefaultDeltaOnRemoval_HeaderFile
