// Created on: 2002-04-29
// Created by: Alexander KARTOMIN (akm)
// Copyright (c) 2002-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef NCollection_HArray1_HeaderFile
#define NCollection_HArray1_HeaderFile

#include <NCollection_Array1.hxx>
#include <NCollection_DefineHArray1.hxx>

//      Declaration of Array1 class managed by Handle

#define NCOLLECTION_HARRAY1(HClassName, Type) DEFINE_HARRAY1(HClassName, NCollection_Array1<Type>)

#endif
