// Created on: 1999-07-29
// Created by: Roman LYGIN
// Copyright (c) 1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _Message_Gravity_HeaderFile
#define _Message_Gravity_HeaderFile

//! Defines gravity level of messages
//! - Trace: low-level details on algorithm execution (usually for debug purposes)
//! - Info: informative message
//! - Warning: warning message
//! - Alarm: non-critical error
//! - Fail: fatal error
enum Message_Gravity
{
  Message_Trace,
  Message_Info,
  Message_Warning,
  Message_Alarm,
  Message_Fail
};

#endif // _Message_Gravity_HeaderFile
