// Created on: 1991-04-11
// Created by: Laurent PAINNOT
// Copyright (c) 1991-1999 Matra Datavision
// Copyright (c) 1999-2014 OPEN CASCADE SAS
//
// This file is part of Open CASCADE Technology software library.
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License version 2.1 as published
// by the Free Software Foundation, with special exception defined in the file
// OCCT_LGPL_EXCEPTION.txt. Consult the file LICENSE_LGPL_21.txt included in OCCT
// distribution for complete text of the license and disclaimer of any warranty.
//
// Alternatively, this file may be used under the terms of Open CASCADE
// commercial license or contractual agreement.

#ifndef _AppParCurves_Constraint_HeaderFile
#define _AppParCurves_Constraint_HeaderFile

//! -   NoConstraint: this point has no constraints.
//! -   PassPoint: the approximation curve passes through this point.
//! -   TangencyPoint: this point has a tangency constraint.
//! -   CurvaturePoint: this point has a curvature constraint.
enum AppParCurves_Constraint
{
  AppParCurves_NoConstraint,
  AppParCurves_PassPoint,
  AppParCurves_TangencyPoint,
  AppParCurves_CurvaturePoint
};

#endif // _AppParCurves_Constraint_HeaderFile
