#ifndef TKTKV3d_PCH_H
#define TKTKV3d_PCH_H

#include <AIS_InteractiveObject.hxx>
#include <PrsMgr_PresentableObject.hxx>
#include <SelectMgr_EntityOwner.hxx>
#include <SelectMgr_SelectableObject.hxx>

#endif // TKTKV3d_PCH_H
